@@ -2,6 +2,7 @@ from enum import Enum
 from urllib.parse import quote_plus
 
 from pymongo import MongoClient
+from pymongo.database import Database
 from pymongo.errors import ServerSelectionTimeoutError, OperationFailure
 
 MAX_SEV_SEL_DELAY = 5
@@ -26,14 +27,13 @@ class Mode(Enum):
     EC2_Instance = 'MONGODB-AWS'
 
 
-def uri_connect(host: str = 'localhost', port: int = 27017, replicaset: str = None, database: str = None,
+def uri_connect(host: str = 'localhost', port: int = 27017, replicaset: str = None,
                 user: str = None, password: str = None, mode: Mode = None,
                 cert_key_file: str = None, ca_file: str = None, session_token: str = None) -> MongoClient:
     """ Connect with the database using different URI authentication methods.
     :param host: The database connection host.
     :param port: The database connection port.
     :param replicaset: The replicaset. If None, then, replicaset is not used.
-    :param database: The MongoDB database to connect.
     :param user: The MongoDB username, access key id or session token. If None, then, username is used.
     :param password: The MongoDB password or secret access key. If None, then, password is used.
     :param mode: The authentication mode. By default, the URL mode is used.
@@ -56,10 +56,7 @@ def uri_connect(host: str = 'localhost', port: int = 27017, replicaset: str = No
     uri += f'@{host}' if user and mode not in [Mode.MONGODB_AWS] else host
     if mode and mode.value:
         uri += f'?authMechanism={mode.value}' + \
-            (f'&authSource={database}' if database else '') + \
             (f'&authMechanismProperties={session_token}' if session_token else '')
-    else:
-        uri += f'?authSource={database}' if database else ''
 
     if cert_key_file:
         if replicaset:
@@ -76,13 +73,12 @@ def uri_connect(host: str = 'localhost', port: int = 27017, replicaset: str = No
     return client
 
 
-def auth_connect(host: str = 'localhost', port: int = 27017, replicaset: str = None, database: str = None,
+def auth_connect(host: str = 'localhost', port: int = 27017, replicaset: str = None,
                  user: str = None, password: str = None, mode: Mode = Mode.SCRAM_SHA_256) -> MongoClient:
     """ Connect with the database using the specified authentication mechanism.
     :param host: The database connection host.
     :param port: The database connection port.
     :param replicaset: The replicaset. If None, then, replicaset is not used.
-    :param database: The MongoDB database to connect.
     :param user: The MongoDB username, access key id or session token. If None, then, username is used.
     :param password: The MongoDB password or secret access key. If None, then, password is used.
     :param mode: The authentication mode. By default, the URL mode is used.
@@ -92,14 +88,8 @@ def auth_connect(host: str = 'localhost', port: int = 27017, replicaset: str = N
         raise ValueError('The mode AUTO is not supported directly by this function.'
                          ' You need to use connect() or connect_database() functions.')
     if replicaset:
-        if database:
-            return MongoClient(host, port, replicaset=replicaset, username=user, password=password, authSource=database,
-                               authMechanism=mode.value, serverSelectionTimeoutMS=MAX_SEV_SEL_DELAY)
         return MongoClient(host, port, replicaset=replicaset, username=user, password=password,
                            authMechanism=mode.value, serverSelectionTimeoutMS=MAX_SEV_SEL_DELAY)
-    if database:
-        return MongoClient(host, port, username=user, password=password, authSource=database,
-                           authMechanism=mode.value, serverSelectionTimeoutMS=MAX_SEV_SEL_DELAY)
     return MongoClient(host, port, username=user, password=password,
                        authMechanism=mode.value, serverSelectionTimeoutMS=MAX_SEV_SEL_DELAY)
 
@@ -119,12 +109,20 @@ def connect(host: str = 'localhost', port: int = 27017, replicaset: str = None,
     :param session_token: The session token to use.
     :return: The MongoDB client.
     """
-    return connect_database(host, port, replicaset, None, user, password, mode, cert_key_file, ca_file, session_token)
+    if not user or mode == Mode.NONE:
+        return uri_connect(host, port, replicaset, mode=Mode.NONE)
+    if mode == Mode.AUTO:
+        return detect_connection(host, port, replicaset, user, password, cert_key_file, ca_file, session_token)
+    if mode in [Mode.MONGODB_X509, Mode.GSSAPI]:
+        return uri_connect(host, port, replicaset, user, None, mode, cert_key_file, ca_file, session_token)
+    if mode in [Mode.SCRAM_SHA_256, Mode.SCRAM_SHA_1]:
+        return auth_connect(host, port, replicaset, user, password, mode)
+    return uri_connect(host, port, replicaset, user, password, mode, cert_key_file, ca_file, session_token)
 
 
 def connect_database(host: str = 'localhost', port: int = 27017, replicaset: str = None, database: str = None,
                      user: str = None, password: str = None, mode: Mode = Mode.AUTO,
-                     cert_key_file: str = None, ca_file: str = None, session_token: str = None) -> MongoClient:
+                     cert_key_file: str = None, ca_file: str = None, session_token: str = None) -> Database:
     """ Connect with the database.
     :param host: The database connection host.
     :param port: The database connection port.
@@ -138,19 +136,13 @@ def connect_database(host: str = 'localhost', port: int = 27017, replicaset: str
     :param session_token: The session token to use.
     :return: The MongoDB client.
     """
-    if not user or mode == Mode.NONE:
-        return uri_connect(host, port, replicaset, database, mode=Mode.NONE)
-    if mode == Mode.AUTO:
-        return detect_connection(host, port, replicaset, database,
-                                 user, password, cert_key_file, ca_file, session_token)
-    if mode in [Mode.MONGODB_X509, Mode.GSSAPI]:
-        return uri_connect(host, port, replicaset, database, user, None, mode, cert_key_file, ca_file, session_token)
-    if mode in [Mode.SCRAM_SHA_256, Mode.SCRAM_SHA_1]:
-        return auth_connect(host, port, replicaset, database, user, password, mode)
-    return uri_connect(host, port, replicaset, database, user, password, mode, cert_key_file, ca_file, session_token)
+    client = connect(host, port, replicaset, user, password, mode, cert_key_file, ca_file, session_token)
+    db = client[database]
+    db.list_collection_names()
+    return db
 
 
-def detect_connection(host: str = 'localhost', port: int = 27017, replicaset: str = None, database: str = None,
+def detect_connection(host: str = 'localhost', port: int = 27017, replicaset: str = None,
                       user: str = None, password: str = None,
                       cert_key_file: str = None, ca_file: str = None, session_token: str = None) -> MongoClient:
     """ Connect with the database detecting the connection method.
@@ -159,7 +151,6 @@ def detect_connection(host: str = 'localhost', port: int = 27017, replicaset: st
     :param replicaset: The replicaset. If None, then, replicaset is not used.
     :param user: The MongoDB username, access key id or session token. If None, then, username is used.
     :param password: The MongoDB password or secret access key. If None, then, password is used.
-    :param database: The MongoDB database to connect.
     :param cert_key_file: The path to the certificate key file.
     :param ca_file: The path to the CA file.
     :param session_token: The session token to use.
@@ -167,10 +158,9 @@ def detect_connection(host: str = 'localhost', port: int = 27017, replicaset: st
     """
     for mode in list(Mode)[2:]:
         try:
-            client = connect_database(host, port, replicaset, database,
-                                      user, password, mode, cert_key_file, ca_file, session_token)
+            client = connect(host, port, replicaset, user, password, mode, cert_key_file, ca_file, session_token)
             client.list_database_names()
             return client
-        except (ServerSelectionTimeoutError, OperationFailure) as e:
+        except (ServerSelectionTimeoutError, OperationFailure):
             pass
     raise OperationFailure('Authentication error')
